Package: inteli
Title: Interval Estimation by Likelihood Method
Version: 0.2.0
Authors@R: c(
    person("Minkyu", "Kim", , "mkim@acr.kr", role = c("aut", "cre")),
    person("Kyun-Seop", "Bae", , "k@acr.kr", role = "aut",
    comment = c(ORCID = "0000-0001-7399-5879"))
    )
Description: Currently used CI method has its limitation when the test statistics are asymmetrical (chi-square test, F-test) or the model functions are non-linear. It can be overcome by using the likelihood functions for the interval estimation. 'inteli' package now supports interval estimation for the mean, variance, variance ratio, binomial distribution, Poisson distribution, odds ratio, risk difference, relative risk and their likelihood function plots. Testing functions are also provided.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 8.0.0
Imports: graphics, stats
NeedsCompilation: no
Packaged: 2026-05-24 16:17:13 UTC; mkim
Author: Minkyu Kim [aut, cre],
  Kyun-Seop Bae [aut] (ORCID: <https://orcid.org/0000-0001-7399-5879>)
Maintainer: Minkyu Kim <mkim@acr.kr>
Repository: CRAN
Date/Publication: 2026-05-24 16:30:02 UTC
Built: R 4.6.0; ; 2026-05-24 18:39:55 UTC; unix
