# Generated by roxygen2: do not edit by hand

export(lib)
export(lim)
export(liod)
export(lipois)
export(lir)
export(lird)
export(lirr)
export(liv)
export(qlib)
export(qlim)
export(qlipois)
export(qliv)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,par)
importFrom(graphics,points)
importFrom(stats,binom.test)
importFrom(stats,nlminb)
importFrom(stats,poisson.test)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,t.test)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,var.test)
